# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: janggi/proto/log.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x16janggi/proto/log.proto\x12\x06janggi\"$\n\x08Location\x12\x0b\n\x03row\x18\x01 \x01(\x05\x12\x0b\n\x03\x63ol\x18\x02 \x01(\x05\"H\n\x04Move\x12 \n\x06origin\x18\x01 \x01(\x0b\x32\x10.janggi.Location\x12\x1e\n\x04\x64\x65st\x18\x02 \x01(\x0b\x32\x10.janggi.Location\"\x99\x01\n\x03Log\x12(\n\rcho_formation\x18\x01 \x01(\x0e\x32\x11.janggi.Formation\x12(\n\rhan_formation\x18\x02 \x01(\x0e\x32\x11.janggi.Formation\x12!\n\x0b\x62ottom_camp\x18\x03 \x01(\x0e\x32\x0c.janggi.Camp\x12\x1b\n\x05moves\x18\x04 \x03(\x0b\x32\x0c.janggi.Move*\x18\n\x04\x43\x61mp\x12\x07\n\x03\x43HO\x10\x00\x12\x07\n\x03HAN\x10\x01*Z\n\tFormation\x12\x12\n\x0eINNER_ELEPHANT\x10\x00\x12\x12\n\x0eOUTER_ELEPHANT\x10\x01\x12\x11\n\rLEFT_ELEPHANT\x10\x02\x12\x12\n\x0eRIGHT_ELEPHANT\x10\x03\x62\x06proto3')

_CAMP = DESCRIPTOR.enum_types_by_name['Camp']
Camp = enum_type_wrapper.EnumTypeWrapper(_CAMP)
_FORMATION = DESCRIPTOR.enum_types_by_name['Formation']
Formation = enum_type_wrapper.EnumTypeWrapper(_FORMATION)
CHO = 0
HAN = 1
INNER_ELEPHANT = 0
OUTER_ELEPHANT = 1
LEFT_ELEPHANT = 2
RIGHT_ELEPHANT = 3


_LOCATION = DESCRIPTOR.message_types_by_name['Location']
_MOVE = DESCRIPTOR.message_types_by_name['Move']
_LOG = DESCRIPTOR.message_types_by_name['Log']
Location = _reflection.GeneratedProtocolMessageType('Location', (_message.Message,), {
  'DESCRIPTOR' : _LOCATION,
  '__module__' : 'janggi.proto.log_pb2'
  # @@protoc_insertion_point(class_scope:janggi.Location)
  })
_sym_db.RegisterMessage(Location)

Move = _reflection.GeneratedProtocolMessageType('Move', (_message.Message,), {
  'DESCRIPTOR' : _MOVE,
  '__module__' : 'janggi.proto.log_pb2'
  # @@protoc_insertion_point(class_scope:janggi.Move)
  })
_sym_db.RegisterMessage(Move)

Log = _reflection.GeneratedProtocolMessageType('Log', (_message.Message,), {
  'DESCRIPTOR' : _LOG,
  '__module__' : 'janggi.proto.log_pb2'
  # @@protoc_insertion_point(class_scope:janggi.Log)
  })
_sym_db.RegisterMessage(Log)

if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _CAMP._serialized_start=302
  _CAMP._serialized_end=326
  _FORMATION._serialized_start=328
  _FORMATION._serialized_end=418
  _LOCATION._serialized_start=34
  _LOCATION._serialized_end=70
  _MOVE._serialized_start=72
  _MOVE._serialized_end=144
  _LOG._serialized_start=147
  _LOG._serialized_end=300
# @@protoc_insertion_point(module_scope)
